=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-09-16 03:32:44 +0000
+++ bzrlib/__init__.py	2005-09-17 18:19:20 +0000
@@ -48,7 +48,8 @@
     from bzrlib.branch import Branch
     
     try:
-        branch = Branch.open(__path__[0])
+        import os
+        branch = Branch.open(os.path.dirname(__path__[0]))
         rh = branch.revision_history()
         if rh:
             return len(rh), rh[-1]

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-09-16 10:45:59 +0000
+++ bzrlib/builtins.py	2005-09-18 18:16:42 +0000
@@ -87,14 +87,32 @@
 
 
 class cmd_cat_revision(Command):
-    """Write out metadata for a revision."""
+    """Write out metadata for a revision.
+    
+    The revision to print can either be specified by a specific
+    revision identifier, or you can use --revision.
+    """
 
     hidden = True
-    takes_args = ['revision_id']
+    takes_args = ['revision_id?']
+    takes_options = ['revision']
     
-    def run(self, revision_id):
+    def run(self, revision_id=None, revision=None):
+        from bzrlib.revisionspec import RevisionSpec
+
+        if revision_id is not None and revision is not None:
+            raise BzrCommandError('You can only supply one of revision_id or --revision')
+        if revision_id is None and revision is None:
+            raise BzrCommandError('You must supply either --revision or a revision_id')
         b = Branch.open_containing('.')
-        sys.stdout.write(b.get_revision_xml_file(revision_id).read())
+        if revision_id is not None:
+            sys.stdout.write(b.get_revision_xml_file(revision_id).read())
+        elif revision is not None:
+            for rev in revision:
+                if rev is None:
+                    raise BzrCommandError('You cannot specify a NULL revision.')
+                revno, rev_id = rev.in_history(b)
+                sys.stdout.write(b.get_revision_xml_file(rev_id).read())
 
 
 class cmd_revno(Command):
@@ -111,21 +129,26 @@
     hidden = True
     takes_args = ['revision_info*']
     takes_options = ['revision']
-    def run(self, revision=None, revision_info_list=()):
+    def run(self, revision=None, revision_info_list=[]):
         from bzrlib.revisionspec import RevisionSpec
 
         revs = []
         if revision is not None:
             revs.extend(revision)
-        for rev in revision_info_list:
-            revs.append(RevisionSpec(revision_info_list))
+        if revision_info_list is not None:
+            for rev in revision_info_list:
+                revs.append(RevisionSpec(rev))
         if len(revs) == 0:
             raise BzrCommandError('You must supply a revision identifier')
 
         b = Branch.open_containing('.')
 
         for rev in revs:
-            print '%4d %s' % rev.in_history(b)
+            revinfo = rev.in_history(b)
+            if revinfo.revno is None:
+                print '     %s' % revinfo.rev_id
+            else:
+                print '%4d %s' % (revinfo.revno, revinfo.rev_id)
 
     
 class cmd_add(Command):
@@ -1187,8 +1210,11 @@
                 if None in revision:
                     raise BzrCommandError(
                         "Merge doesn't permit that revision specifier.")
-                base = [branch, revision[0].in_history(branch).revno]
-                other = [branch, revision[1].in_history(branch).revno]
+                from bzrlib.branch import Branch
+                b = Branch.open(branch)
+
+                base = [branch, revision[0].in_history(b).revno]
+                other = [branch, revision[1].in_history(b).revno]
 
         try:
             merge(other, base, check_clean=(not force), merge_type=merge_type)
@@ -1222,10 +1248,13 @@
             if len(file_list) == 0:
                 raise BzrCommandError("No files specified")
         if revision is None:
-            revision = [-1]
+            revno = -1
         elif len(revision) != 1:
             raise BzrCommandError('bzr revert --revision takes exactly 1 argument')
-        merge(('.', revision[0]), parse_spec('.'),
+        else:
+            b = Branch.open_containing('.')
+            revno = revision[0].in_history(b).revno
+        merge(('.', revno), parse_spec('.'),
               check_clean=False,
               ignore_zero=True,
               backup_files=not no_backup,

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2005-09-15 15:16:12 +0000
+++ bzrlib/revisionspec.py	2005-09-18 18:16:42 +0000
@@ -110,10 +110,16 @@
         if spec is None:
             return object.__new__(RevisionSpec, spec)
 
+        if isinstance(spec, RevisionSpec):
+            return object.__new__(type(spec), spec)
+
         try:
             spec = int(spec)
         except ValueError:
             pass
+        except TypeError:
+            raise TypeError('Unexpected type, got %s, %r' % (type(spec), spec))
+
 
         if isinstance(spec, int):
             return object.__new__(RevisionSpec_int, spec)
@@ -128,9 +134,12 @@
             raise TypeError('Unhandled revision type %s' % spec)
 
     def __init__(self, spec):
-        if self.prefix and spec.startswith(self.prefix):
-            spec = spec[len(self.prefix):]
-        self.spec = spec
+        if isinstance(spec, RevisionSpec):
+            self.spec = spec.spec
+        else:
+            if self.prefix and spec.startswith(self.prefix):
+                spec = spec[len(self.prefix):]
+            self.spec = spec
 
     def _match_on(self, branch, revs):
         return RevisionInfo(branch, 0, None)
@@ -157,13 +166,21 @@
                               self.prefix or '',
                               self.spec)
 
+    def __eq__(self, other):
+        if isinstance(other, RevisionSpec):
+            return self.spec == other.spec
+        return other == self.spec
+
 
 # private API
 
 class RevisionSpec_int(RevisionSpec):
     """Spec is a number.  Special case."""
     def __init__(self, spec):
-        self.spec = int(spec)
+        if isinstance(spec, RevisionSpec):
+            self.spec = int(spec.spec)
+        else:
+            self.spec = int(spec)
 
     def _match_on(self, branch, revs):
         if self.spec < 0:

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-09-16 12:59:36 +0000
+++ bzrlib/selftest/__init__.py	2005-09-17 16:26:16 +0000
@@ -338,6 +338,7 @@
                    'bzrlib.selftest.testbranch',
                    'bzrlib.selftest.testremotebranch',
                    'bzrlib.selftest.testrevision',
+                   'bzrlib.selftest.test_revision_info',
                    'bzrlib.selftest.test_merge_core',
                    'bzrlib.selftest.test_smart_add',
                    'bzrlib.selftest.testdiff',

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-09-16 13:28:33 +0000
+++ bzrlib/selftest/blackbox.py	2005-09-19 04:43:28 +0000
@@ -74,10 +74,22 @@
         f = file('.bzr/email', 'wt')
         f.write('Branch Identity <branch@identi.ty>')
         f.close()
+        bzr_email = os.environ.get('BZREMAIL')
+        if bzr_email is not None:
+            del os.environ['BZREMAIL']
         whoami = self.runbzr("whoami",backtick=True)
         whoami_email = self.runbzr("whoami --email",backtick=True)
         self.assertTrue(whoami.startswith('Branch Identity <branch@identi.ty>'))
         self.assertTrue(whoami_email.startswith('branch@identi.ty'))
+        # Verify that the environment variable overrides the value 
+        # in the file
+        os.environ['BZREMAIL'] = 'Different ID <other@environ.ment>'
+        whoami = self.runbzr("whoami",backtick=True)
+        whoami_email = self.runbzr("whoami --email",backtick=True)
+        self.assertTrue(whoami.startswith('Different ID <other@environ.ment>'))
+        self.assertTrue(whoami_email.startswith('other@environ.ment'))
+        if bzr_email is not None:
+            os.environ['BZREMAIL'] = bzr_email
 
     def test_invalid_commands(self):
         self.runbzr("pants", retcode=1)
@@ -148,6 +160,13 @@
         os.rmdir('revertdir')
         self.runbzr('revert')
 
+        file('hello', 'wt').write('xyz')
+        self.runbzr('commit -m xyz hello')
+        self.runbzr('revert -r 1 hello')
+        self.check_file_contents('hello', 'foo')
+        self.runbzr('revert hello')
+        self.check_file_contents('hello', 'xyz')
+
     def test_mv_modes(self):
         """Test two modes of operation for mv"""
         from bzrlib.branch import Branch
@@ -193,15 +212,14 @@
         output = self.runbzr('diff -r last:3..last:1', backtick=1)
         self.assert_('\n+baz' in output)
 
-    def test_revert(self):
+    def test_diff(self):
         self.example_branch()
-        file('hello', 'wt').write('bar')
-        file('goodbye', 'wt').write('qux')
-        self.runbzr('revert hello')
-        self.check_file_contents('hello', 'foo')
-        self.check_file_contents('goodbye', 'qux')
-        self.runbzr('revert')
-        self.check_file_contents('goodbye', 'baz')
+        file('hello', 'wt').write('hello world!')
+        self.runbzr('commit -m fixing hello')
+        output = self.runbzr('diff -r 2..3', backtick=1)
+        self.assert_('\n+hello world!' in output)
+        output = self.runbzr('diff -r last:3..last:1', backtick=1)
+        self.assert_('\n+baz' in output)
 
     def test_merge(self):
         from bzrlib.branch import Branch

=== added file 'bzrlib/selftest/test_revision_info.py'
--- bzrlib/selftest/test_revision_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_revision_info.py	2005-09-17 16:26:16 +0000
@@ -0,0 +1,103 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.errors import BzrCommandError, NoSuchRevision
+from bzrlib.branch import Branch
+from bzrlib.revisionspec import RevisionSpec
+
+class TestRevisionInfo(TestCaseInTempDir):
+    def test_revision_info(self):
+        """Test that 'bzr revision-info' reports the correct thing.
+        """
+
+        b = Branch.initialize('.')
+
+        b.commit('Commit one', rev_id='a@r-0-1')
+        b.commit('Commit two', rev_id='a@r-0-2')
+        b.commit('Commit three', rev_id='a@r-0-3')
+
+        def check(output, *args):
+            """Verify that the expected output matches what bzr says.
+            
+            The output is supplied first, so that you can supply a variable
+            number of arguments to bzr.
+            """
+            self.assertEquals(self.backtick(['bzr'] + list(args)), output)
+
+        # Make sure revision-info without any arguments throws an exception
+        self.assertRaises(BzrCommandError, self.run_bzr, 'revision-info')
+
+        values = {
+            1:'   1 a@r-0-1\n',
+            2:'   2 a@r-0-2\n',
+            3:'   3 a@r-0-3\n'
+        }
+
+        # Check the results of just specifying a numeric revision
+        check(values[1], 'revision-info', '1')
+        check(values[2], 'revision-info', '2')
+        check(values[3], 'revision-info', '3')
+        check(values[1]+values[2], 'revision-info', '1', '2')
+        check(values[1]+values[2]+values[3], 'revision-info', '1', '2', '3')
+        check(values[2]+values[1], 'revision-info', '2', '1')
+
+        # Check as above, only using the '--revision' syntax
+        
+        check('   1 a@r-0-1\n', 'revision-info', '-r', '1')
+        check('   2 a@r-0-2\n', 'revision-info', '--revision', '2')
+        check('   3 a@r-0-3\n', 'revision-info', '-r', '3')
+        check('   1 a@r-0-1\n   2 a@r-0-2\n', 'revision-info', '-r', '1..2')
+        check('   1 a@r-0-1\n   2 a@r-0-2\n   3 a@r-0-3\n'
+                , 'revision-info', '-r', '1..2..3')
+        check('   2 a@r-0-2\n   1 a@r-0-1\n', 'revision-info', '-r', '2..1')
+
+        # Now try some more advanced revision specifications
+        
+        check('   1 a@r-0-1\n', 'revision-info', '-r', 'revid:a@r-0-1')
+        check('   2 a@r-0-2\n', 'revision-info', '--revision', 'revid:a@r-0-2')
+
+    def test_cat_revision(self):
+        """Test bzr cat-revision.
+        """
+        b = Branch.initialize('.')
+
+        b.commit('Commit one', rev_id='a@r-0-1')
+        b.commit('Commit two', rev_id='a@r-0-2')
+        b.commit('Commit three', rev_id='a@r-0-3')
+
+        revs = {
+            1:b.get_revision_xml_file('a@r-0-1').read(),
+            2:b.get_revision_xml_file('a@r-0-2').read(),
+            3:b.get_revision_xml_file('a@r-0-3').read()
+        }
+
+        def check(output, *args):
+            self.assertEquals(self.backtick(['bzr'] + list(args)), output)
+
+        check(revs[1], 'cat-revision', 'a@r-0-1')
+        check(revs[2], 'cat-revision', 'a@r-0-2')
+        check(revs[3], 'cat-revision', 'a@r-0-3')
+
+        check(revs[1], 'cat-revision', '-r', '1')
+        check(revs[2], 'cat-revision', '-r', '2')
+        check(revs[3], 'cat-revision', '-r', '3')
+
+        check(revs[1], 'cat-revision', '-r', 'revid:a@r-0-1')
+        check(revs[2], 'cat-revision', '-r', 'revid:a@r-0-2')
+        check(revs[3], 'cat-revision', '-r', 'revid:a@r-0-3')
+

=== modified file 'bzrlib/selftest/testrevisionnamespaces.py'
--- bzrlib/selftest/testrevisionnamespaces.py	2005-09-16 03:38:22 +0000
+++ bzrlib/selftest/testrevisionnamespaces.py	2005-09-17 18:10:13 +0000
@@ -74,3 +74,16 @@
                           'b@r-0-4')
         self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
                           'b@r-0-4')
+
+
+        # Make sure you can create a revision spec from another revision spec
+        spec = RevisionSpec('revid:a@r-0-4')
+        self.assertEquals(spec, RevisionSpec(spec))
+
+        spec = RevisionSpec('1')
+        self.assertEquals(spec, RevisionSpec(spec))
+
+        spec_1 = RevisionSpec(1)
+        self.assertEquals(spec_1, RevisionSpec(spec_1))
+        self.assertEquals(spec, spec_1)
+

